@@ -14,13 +14,13 @@ try:
 except ImportError:
     from io import StringIO
 #import numpy as np  
-#import pandas as pd
+import pandas as pd
 
 def make_plot(*args, **kwargs):
-    subplot = {'subtype': 'plot',
-            'cmd': [] }
-    subplot["attribute"] = collections.OrderedDict()
 
+    subplot = {'data': None,
+            'subtype': 'plot',
+            'cmd': [] }
     subplot["attribute"] = collections.OrderedDict()
 
     for v in args:
@@ -30,7 +30,9 @@ def make_plot(*args, **kwargs):
     return subplot
 
 def make_splot(*args, **kwargs):
-    subplot = {'subtype': 'splot',
+
+    subplot = {'data': None,
+            'subtype': 'splot',
             'cmd': [] }
     subplot["attribute"] = collections.OrderedDict()
 
@@ -40,35 +42,56 @@ def make_splot(*args, **kwargs):
         subplot["attribute"][k] = v
     return subplot
 
+def make_plot_data (data, *args, **kwargs):
+    subplot = {'data': data,
+            'subtype': 'plot',
+            'cmd': []}
+    subplot["attribute"] = collections.OrderedDict()
+
+    for v in args:
+        subplot["cmd"].append(v)
+    for k,v in kwargs.items():
+        subplot["attribute"][k] = v
+    return subplot
+
+def make_splot_data (data, *args, **kwargs):
+    subplot = {'data': data,
+            'subtype': 'splot',
+            'cmd': []}
+    subplot["attribute"] = collections.OrderedDict()
+
+    for v in args:
+        subplot["cmd"].append(v)
+    for k,v in kwargs.items():
+        subplot["attribute"][k] = v
+    return subplot
+
 def multiplot(*args, **kwargs):
-    '''
-    @args: the subplot object list;
-    @kwargs: the setting options that need to be set before call plot;
-    '''
     g = Gnuplot()
 
-    for k,v in kwargs.items():
-        g.cmd('set %s %s' %(k, v))
+    g.set(**kwargs)
     if 'multiplot' not in kwargs.keys():
         g.cmd('set multiplot')
 
-    #print(args)
     for subplot in args:
         g.set(**subplot["attribute"])
-        cmd = subplot["cmd"]
         c = subplot["subtype"]
-        for cmd in subplot["cmd"]:
-            c += ' %s,' %(cmd)
-        #print("log multiplot(): ", c.rstrip(','))
-        g.cmd(c.rstrip(','))
-        # multiplot automatically unset the following setting:
+
+        if (subplot["data"] is not None):
+            # Conver the data to string format:
+            if isinstance(subplot["data"], pd.DataFrame):
+                content = subplot["data"].to_csv(sep = ' ')
+            else:
+                content = str(subplot["data"])
+            g.__call__('$DataFrame << EOD\n%s\nEOD' %(content))
+            for item in subplot["cmd"]:
+                c += ' $DataFrame %s,\\\n' %(item)
+        else:
+            for item in subplot["cmd"]:
+                c += ' %s,\\\n' %(item)
+        cmd = c.rstrip(",\\\r\n")
+        g.cmd(cmd + '\n')
         g.unset('for [i=1:200] label i')
-        #g.unset('for [i=1:200] label i',
-        #        'title',
-        #        'xtics', 'x2tics', 'ytics', 'y2tics', 'ztics', 'cbtics',
-        #        'xlabel', 'x2label', 'ylabel', 'y2label', 'zlabel', 'cblabel',
-        #        'xrange', 'x2range', 'yrange', 'y2range', 'zrange', 'cbrange',
-        #        'rrange', 'trange', 'urange', 'vrange')
     g.reset()
 
 def plot(*args, **kwargs):
@@ -87,7 +110,32 @@ def __gnuplot(plot_cmd, *args, **kwargs):
     c = plot_cmd
     for cmd in args:
         c += ' %s,' %(cmd)
-    #print(c.rstrip(','))
+    g.cmd(c.rstrip(','))
+    g.reset()
+
+
+def plot_data(data, *args, **kwargs):
+    __gnuplot_data(data, "plot", *args, **kwargs)
+
+def splot_data(data, *args, **kwargs):
+    __gnuplot_data(data, "splot", *args, **kwargs)
+
+def __gnuplot_data(data, plot_cmd, *args, **kwargs):
+    g = Gnuplot()
+
+    # kwargs input:
+    g.set(**kwargs)
+
+    # Conver the data to string format:
+    if isinstance(data, pd.DataFrame):
+        content = data.to_csv(sep = ' ')
+    else:
+        content = str(data)
+    g.__call__('$DataFrame << EOD\n%s\nEOD' %(content))
+    c = plot_cmd
+    for cmd in args:
+        c += ' $DataFrame %s,' %(cmd)
+    #print(c)
     g.cmd(c.rstrip(','))
     g.reset()
 
@@ -101,7 +149,7 @@ class Gnuplot(object):
     def __init__(self, *args, log = False, **kwargs):
         '''
         *args: The flag parameter in gnuplot
-        log: If print the gnuplot log
+        log: If show the gnuplot log
         **kwargs: the flag that need to be set. You can also set them in the set() function.
         '''
         self.gnuplot = subprocess.Popen(['gnuplot','-p'], shell=True, stdin=subprocess.PIPE)
@@ -128,7 +176,6 @@ class Gnuplot(object):
         '''
         commands = []
         for cmd in args:
-            #print StringIO(cmd.strip()).readlines()
             cmd = filter(lambda x: (x.strip()) and (x.strip()[0] != '#'),
                     StringIO(cmd.strip()).readlines())
             # remove the leading or trailing \r\n
@@ -174,15 +221,12 @@ class Gnuplot(object):
                  contain an iteration clause, so the arg could be list.
         '''
         for v in args:
-            #print('set %s' %(v))
             self.cmd('set %s' %(v))
         for k,v in kwargs.items():
             if isinstance(v, list):
                 for i in v:
-                    #print('set %s %s' %(k, i))
                     self.cmd('set %s %s' %(k, i))
             else:
-                #print('set %s %s' %(k, v))
                 if (v == None):
                     self.cmd('unset %s' %(k))
                 else:
@@ -210,13 +254,51 @@ class Gnuplot(object):
         cmd = c.rstrip(',')
         self.cmd(cmd + '\n')
 
-    # print function couldn't be compiled.
-    #def print(self, *items):
-    #    c = 'print'
-    #    for item in items:
-    #        c = c + " " + item + ","
-    #    cmd = c.rstrip(',')
-    #    self.cmd(cmd + '\n')
+    def plot_data(self, data, *items, **kwargs):
+        '''
+        data: The data that need to be plotted. It's either the string of list
+        or the Pnadas Dataframe, if it's Pnadas Dataframe it would be converted
+        to string by data.to_csv(sep = ' ')
+        *items: The list of plot command;
+        **kwargs: The options that would be set before the plot command.
+        '''
+        self.set(**kwargs)
+        c = 'plot'
+
+        if isinstance(data, pd.DataFrame):
+            content = data.to_csv(sep = ' ')
+        else:
+            content = str(data)
+
+        #self.cmd('$DataFrame << EOD\n%s\nEOD' %(content))
+        self.__call__('$DataFrame << EOD\n%s\nEOD' %(content))
+        for item in items:
+            c += ' $DataFrame %s,\\\n' %(item)
+        cmd = c.rstrip(",\\\r\n")
+        self.cmd(cmd + '\n')
+
+    def splot_data(self, data, *items, **kwargs):
+        '''
+        data: The data that need to be plotted. It's either the string of list
+        or the Pnadas Dataframe, if it's Pnadas Dataframe it would be converted
+        to string by data.to_csv(sep = ' ')
+        *items: The list of plot command;
+        **kwargs: The options that would be set before the plot command.
+        '''
+        self.set(**kwargs)
+        c = 'splot'
+
+        if isinstance(data, pd.DataFrame):
+            content = data.to_csv(sep = ' ')
+        else:
+            content = str(data)
+
+        #self.cmd('$DataFrame << EOD\n%s\nEOD' %(content))
+        self.__call__('$DataFrame << EOD\n%s\nEOD' %(content))
+        for item in items:
+            c += ' $DataFrame %s,' %(item)
+        cmd = c.rstrip(',')
+        self.cmd(cmd + '\n')
 
     def splot(self, *items, **kwargs):
         self.set(**kwargs)
@@ -252,12 +334,10 @@ class Gnuplot(object):
     def __getitem__(self, name): return self.__dict__.get(name.lower(), None)
 
     def __setitem__(self, name, value):
-        #print name,value
         self.cmd('set %s %s\n' %(name, value))
 
     def __call__(self, s):
         """Send a command string to gnuplot, followed by newline."""
-        #print(s)
         cmd = s + '\n'
         self.write(cmd.encode('utf-8'))
         self.flush()
